///////////////////////////////////////////////////////////////////////////
//
// VisualDSP++ 3.5 "Flash Programmer" flash driver for use to
// programm SPI flash/EEPROM devices.
//
// (C) Copyright 2006 - Analog Devices, Inc.  All rights reserved.
//
// Basic understanding of the VisualDSP++ 3.5 flash programmer tool, also 
// called "plug-in", is required.
//
// Note: Due to the fact that the plug-in accepts to open intel hex files only
// 		 SPI programming works just for VisualDSP++ version 3.5 and higher!
//
// 		 Ensure that the option intel hex is chosen at the register LOAD
//       under Project Options!
// 
// In general, when the flash programming tool "Tools->Flash Programmer" 
// has been opened a driver file must be loaded. Loading a driver such as this
// one causes VisualDSP++ to upload the driver dxe file to the processor,
// to set a breakpoint at the label "AFP_BreakReady" and to start the processor.
// When the processor halts at the breakpoint the control went back to the 
// flash plug-in. In the box "Load File" a loader file "*.ldr" of a project
// must be selected to program a SPI device.
//
// Note: As a standard flash needs to be erased before programming please 
//       make use of the erase options in surrounding boxes. 
//
// Pressing the button "Load File" will cause the plug-in to access the loader 
// file and to fill the buffer "AFP_Buffer" with the first portion of data.
// Also, the number of words counted in bytes as well as the SPI start address 
// will be stored in certain variables.
//
// The contents of AFP_Buffer will be written to the SPI and sequentially the 
// AFP_Buffer will be filled with the next portion of the loader file. After 
// the whole loader file has been transferred to the SPI the processor halts 
// at the breakpoint as discussed above. 
//
// 
//////////////////////////////////////////////////////////////////////////////


//--------------------------------------------------------------------------//
// Include/Header files														//
//--------------------------------------------------------------------------//
#include <sys\platform.h>
#include <sys\exception.h>
#include "Errors.h"
#include "SPI_Device_Settings.h"
#include <cdefBF537.h>
#include <stdio.h>
#include <ccblkfn.h>


//--------------------------------------------------------------------------//
// Defines																	//
//--------------------------------------------------------------------------//
#define 		asynchronous_SPORT

// Peripheral Interrupt IDs for various interrupts at SIC_IMASK
#define SPORT1_RX_PID	   			7
#define SPORT1_TX_PID	   			8	
#define SPORT1_ERR_PID				2
#define DMA_ERR_PID					1

#define SPI_ADRS_ARM_INTERRUPTS     (0x00<<8)
#define SPI_ADRS_ARM_INT_EN		    (0x04<<8)

#define SPI_ADRS_HOST_INTERRUPTS    (0x08<<8)
#define SPI_ADRS_HOST_INT_EN	    (0x0c<<8)
#define SPI_ADRS_HOST_INT_ACK	    (0x10<<8)

#define SPI_ADRS_GEN_PURP_1   	    (0x14<<8)
#define SPI_ADRS_GEN_PURP_2   	    (0x18<<8)

#define SPI_ADRS_DEV_CTRL_STAT   	(0x26<<8)    // high word

#define SPI_ADRS_DMA_DATA      	    (0x28<<8)

#define SPI_ADRS_DMA_WRITE_CTRL     (0x2c<<8)
#define SPI_ADRS_DMA_WRITE_LEN      (0x2e<<8)
#define SPI_ADRS_DMA_WRITE_BASE     (0x30<<8)

#define SPI_ADRS_DMA_READ_CTRL      (0x34<<8)
#define SPI_ADRS_DMA_READ_LEN       (0x36<<8)
#define SPI_ADRS_DMA_READ_BASE      (0x38<<8)


// Note that these defines are used by the plug-in also. Deleting these defines
// can lead to Plug-in crashs. 
#define TRUE			0x1
#define FALSE			0x0
#define BUFFER_SIZE		0x600
#define NUM_SECTORS		4
#define SECTORBASE		0

typedef unsigned char 	u8;
typedef unsigned short 	u16;
typedef unsigned int 	u32;

typedef struct DMA_1D_Array_Desc 
{
	u16	START_ADDR_L;
	u16	START_ADDR_H;
    u16 CONFIG;
    u16	X_CNT;
    u16	X_MOD;
} DMA_1D_Array_Desc_t;

typedef struct sm_sport_desc						// struct holding descriptors for a sport_read
{
	DMA_1D_Array_Desc_t MOSI_regs_addr;				// MOSI writes one 16-bit target register address 
	DMA_1D_Array_Desc_t MOSI_regs_value;			// MOSI writing register value (or dummy write in case of sm_sport_read)
	DMA_1D_Array_Desc_t MOSI_regs_value2;			// MOSI writing register value2 (or dummy write in case of sm_sport_read)	
	DMA_1D_Array_Desc_t MISO_regs_addr;				// MISO desc when MOSI writes 16-bit target reg addr (always dummy read)
	DMA_1D_Array_Desc_t MISO_regs_value;			// MISO reads register value (or dummy read in case of sm_sport_write)
	u16 dummy_read;									// place holder for dummy_read data
	u16 dummy_write;								// place holder for dummy_write data
}sm_sport_desc_t;


// For the purpose of erase single sectors of the SPI flash the right sector
// must be chosen. This structure will be taken by the plug-in to display memory space.
typedef struct _SECTORLOCATION
{
	long lStartOff;
	long lEndOff;
	
}SECTORLOCATION;

// Flash Programmer commands
// The VisualDSP++ flash progammer plug-in "Flash Programmer Tool" will send commands
// to this driver. These commands will be taken to execute the corresponding C function
// The commands are used at the switch case below.
// Note: As the commands are used by the plug-in deleting or changing can lead to misfuction.
typedef enum
{
	NO_COMMAND,		// 0
	GET_CODES,		// 1
	RESET,			// 2
	WRITE,			// 3
	FILL,			// 4
	ERASE_ALL,		// 5
	ERASE_SECT,		// 6
	READ,			// 7
	GET_SECTNUM,	// 8
}enProgCmds;

//--------------------------------------------------------------------------//
// Variables																//
//--------------------------------------------------------------------------//
sm_sport_desc_t sm_sport_write_8bits_desc, sm_sport_read_multiple_8bits_desc;
sm_sport_desc_t spi_wren_desc, spi_se_desc, sm_sport_read_8bits_desc;

//--------------------------------------------------------------------------//
// Prototypes																//
//--------------------------------------------------------------------------//
EX_INTERRUPT_HANDLER(SPORT1_ISR);
EX_INTERRUPT_HANDLER(HW_ERR_ISR);
EX_INTERRUPT_HANDLER(SYS_ERR_ISR);
EX_INTERRUPT_HANDLER(EXCEP_ISR);
void Init_PLL(void);	
ERROR_CODE EraseFlash(void);
ERROR_CODE GetCodes(void);
ERROR_CODE ResetFlash(void);
ERROR_CODE WriteData(unsigned long ulStart, long lCount, long lStride, int *pnData);
ERROR_CODE ReadData(unsigned long ulStart, long lCount, long lStride, int *pnData);
ERROR_CODE EraseBlock(int nBlock);
ERROR_CODE FillData( unsigned long ulStart, long lCount, long lStride, int *pnData );
ERROR_CODE WriteFlash (unsigned long ulStartAddr, long lTransferCount, long lModify, int *iDataSource, long *lWriteCount);
ERROR_CODE Wait_For_Status(char Statusbit);
ERROR_CODE GetSectorNumber( unsigned long ulOffset, int *pnSector );
ERROR_CODE Wait_For_nStatus(void);
void Init_Descriptors(void);
void Init_SPORT(void);
char ReadStatusRegister(void);
void SendSingleCommand(u8 addr);
void SectorErase(u8 opcode, u8 *addr);
int  sm_spi_read_8bits(u8 addr, u8 *data, u32 len);
void sm_spi_write_8bits(u8 addr, u8 *data, u8 *dataout, u32 len, u32 modify);
void sm_spi_read_multiple_8bits(u8 opcode, u8 *addr, u8 *datain, u32 len); 




// Global data for use with the VisualDSP++ plug-in
// The plug-in will modify some AFP variables to control this driver
char 			*AFP_Title = "ADSP-BF537 EZ-KIT Lite";	// This title will be displayed in the plug-in
char 			*AFP_Description = SPI_Description;		// The description will also be displayed in the plug-in. See the file "SPI_Device_Settings.h".
enProgCmds 		AFP_Command = NO_COMMAND;				// AFP_Command is dedicated to control program flow. Look at the switch case below.
int 			AFP_ManCode = 0x0;						// Some devices have Manufacturer codes to autodetect the device
int 			AFP_DevCode = 0x0;						// Some devices have device codes to autodetect the device
unsigned long 	AFP_Offset = 0;							// AFP_Offset holds the start address to the SPI device
int 			i, *AFP_Buffer;							// The plug-in copies the content of the *.ldr file to the buffer in small portions
long 			AFP_Size = BUFFER_SIZE;					// The AFP_Buffer size can be modified.
long 			AFP_Count = -1;							// The plug-in copies the number of transfers to be executed to the AFP_Count 
long 			AFP_Stride = -1;						// For the plug-in "Fill" function a stride can optionally be inserted
int 			AFP_NumSectors = 0;						// The number of SPI sectors are given for sector erase function
long 			AFP_SectorSize = 0;						// The multiplier factor from sector to sector must be given
int 			AFP_Sector = -1;						// In case of sector erase the right sector must be chosen and hold in here
int 			AFP_Error = 0;							// contains last error encountered
bool 			AFP_Verify = FALSE;						// verify writes or not
long 			AFP_StartOff = 0x0;						// sector start offset
long 			AFP_EndOff = 0x0;						// sector end offset
int				AFP_FlashWidth = 0x8;					// width of the flash device taken
int 			*AFP_SectorInfo;
SECTORLOCATION SectorInfo[NUM_SECTORS];


// exit flag
bool bExit = FALSE;

//--------------------------------------------------------------------------//
// Function:	main														//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description:	main routine												//
//--------------------------------------------------------------------------//
main()
{
 	int i = 0;
 	
	// By making AFP_Buffer as big as possible the plug-in can send and
	// receive more data at a time making the data transfer quicker.
	//
	// By allocating it on the heap the compiler does not create an
	// initialized array therefore making the driver image smaller
	// and faster to load.
	//
	// We have modified the linker description file (LDF) so that the heap
	// is large enough to store BUFFER_SIZE elements at this point
	
	// The AFP_Buffer gets filled by the programmer plug-in. The data the plug-in fetches
	// from the *.ldr file will be stored in the AFP_Buffer byte wise to avoid gaps in the buffer.
	// The verify command uses the buffer to store the data read from the SPI flash for comparison.
	// The fill command uses the first address of the buffer to hold the value to be filled.
	
	// -In case of loading the code to the SPI flash the plug-in will fill the buffer automatically 
	//  so that this driver just has to fetch the data from the buffer and transfer it to the SPI flash.
	// -In case of reading the data from the SPI flash (plug-in command "VERIFY") this driver just has 
	//  to fetch the data from the SPI flash and store it in the buffer.


	Init_PLL();		
	GetCodes();		// Instead of getting manufacturer codes this driver does organize the 

	
	AFP_Buffer = (int *)malloc(BUFFER_SIZE);
                	// memory stucture "sectors" in the function GetCodes().
	
	// AFP_Buffer will be NULL if we could not allocate storage for the
	// buffer
	if ( AFP_Buffer == NULL )
	{
		// tell GUI that our buffer was not initialized
		AFP_Error = BUFFER_IS_NULL;
	}
	
	Init_Descriptors();
	Init_SPORT();
	
	AFP_SectorInfo = (int*)&SectorInfo[0];	//Points to the beginnig of the sector table.

	// The while loop below is inserted to give the programmer plug-in control over this driver

	// The plug-in will set a breakpoint at "AFP_BreakReady" forcing the processor to halt.
	// After the plug-in starts the processor again the function which is selected by AFP_Command will be executed.
	// After execution the function the processor halts again at the breadpoint at "AFP_BreakReady".
	// Now the plug-in can place the next command in AFP_Command and start the processor again. 
		
	while ( !bExit )
	{
		asm("AFP_BreakReady:");	// After loading the driver to the processor it halt at this point.
				
		if ( FALSE )
			asm("jump AFP_BreakReady;");

		// switch on the command
 		switch ( AFP_Command )
		{
			//In this driver GetCodes does create the sector table
			case GET_CODES:
				AFP_Error = GetCodes();
				break;

			// The plug-in offers to reset the flash. This function will send a "write disable command to the SPI flash.
			//  Additionally, it checks the status register for the write enable bit "WEL" to be cleared.
			case RESET:
				AFP_Error = ResetFlash();
				break;

			// By taking the parameters the function "WriteData" will program the flash.
			// AFP_Offset points to the SPI destination address, AFP_Count holds the number of transfers to execute.
			// AFP_Stride holds the address increment and finally AFP_Buffer points to the source buffer.
			case WRITE:
				AFP_Error = WriteData( AFP_Offset, AFP_Count, AFP_Stride, AFP_Buffer );
				break;

			// As the programmer plug-in allows to simply fill the SPI flash with any value the 
			// "FILL" function includes the code. The AFP_Offset points to the destination while the AFP_Count 
			// holds the number of locations where the value will be placed. AFP_Stride holds the increment
			// while AFP_Buffer points to the memory source.
			case FILL:
				AFP_Error = FillData( AFP_Offset, AFP_Count, AFP_Stride, AFP_Buffer );
				break;

			// Pushing the "Erase" button on the programmer plug-in the driver executes an erase all command.
			case ERASE_ALL:
		#ifdef	SPI_FLASH	//just flashs can be erased!
				AFP_Error = EraseFlash();
		#endif
				break;

			// Pushing the "Erase Sectors" button on the programmer plug-in makes the driver to take the value from
			// AFP_Sector and sends the erase sector command followed by the start address of the dedicated sector.
			// The AFP_Sector gets filled by running the function GET_SECTNUM driven by the plug-in

			case ERASE_SECT:
		#ifdef	SPI_FLASH	//just flashs can be erased!
				AFP_Error = EraseBlock( AFP_Sector );
		#endif
				break;

			// In order to verify the data that has been programmed to the flash the data must been read.
			// This function will be entered when pushing the Verify button.
			case READ:
				AFP_Error = ReadData( AFP_Offset, AFP_Count, AFP_Stride, AFP_Buffer );
				break;

			// Based on the address where to start programming this sector must be erased before. 
			// The function GET_SECTNUM will return the sector number where the start address points to.
			// The sector number will than be taken to run the EraseBlock function.
			case GET_SECTNUM:

				AFP_Error = GetSectorNumber( AFP_Offset, &AFP_Sector );
			break;
			
			// no command or unknown command do nothing
			case NO_COMMAND:
				default:
				// set our error
				AFP_Error = UNKNOWN_COMMAND;
				break;
		}

		// clear the command
		AFP_Command = NO_COMMAND;
	}

// free the buffer if we were able to allocate one
	if ( AFP_Buffer )
		free( AFP_Buffer );
	// all done
	return 0;
}

//--------------------------------------------------------------------------//
// Function:	Init_SPORT													//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description:	This function sets up the SPORT								//
//--------------------------------------------------------------------------//
void Init_SPORT(void)
{
    *pSIC_IMASK = 	  SIC_MASK(SPORT1_TX_PID) 			// SPORT1_TX
					| SIC_MASK(SPORT1_RX_PID)			// SPORT1_RX
#if 0
					| SIC_MASK(SPORT1_ERR) 				// SPORT1 Errors
#endif
					| SIC_MASK(DMA_ERR);				// DMA error

	register_handler_ex (ik_exception, EXCEP_ISR, EX_INT_ENABLE);		// assign ISR to system exception
	register_handler_ex (ik_hardware_err, HW_ERR_ISR, EX_INT_ENABLE);	// assign ISR to HW_ERR
	register_handler_ex (ik_ivg7, SYS_ERR_ISR, EX_INT_ENABLE);			// assign ISR to System Error interrupt
	register_handler_ex (ik_ivg9, SPORT1_ISR, EX_INT_ENABLE);			// assign ISR to SPORT0_TX DMA interrupt

	*pPORTG_FER 		= (0x3F)<<10;
	*pPORT_MUX 			= PGTE | PGRE;
	
	*pSPORT1_TCLKDIV	= 0;							// SPORT CLK = SCLK/2 (45MHz)
	*pSPORT1_TFSDIV		= 0x7;							// SPORT TFS every 8 SPORT CLK cycles 
	*pSPORT1_TCR2		= SLEN(0x7);					// TX word length 8 bit
	*pSPORT1_TCR1		= TCKFE | LATFS	| LTFS | TFSR | ITFS | ITCLK;  

	*pSPORT1_RCLKDIV	= 0;							// SPORT CLK = SCLK/2 (45MHz)
	*pSPORT1_RFSDIV		= 0x7;							// SPORT RFS every 8 SPORT CLK cycles 
	*pSPORT1_RCR2		= SLEN(0x7);					// RX word length 8 bit
	*pSPORT1_RCR1		= RCKFE | LARFS	| LRFS | RFSR;
}


//--------------------------------------------------------------------------//
// Function:	Init_Descriptors											//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description:	This function initializes the descriptor structures			//
//--------------------------------------------------------------------------//
void Init_Descriptors(void)
{
    sm_sport_desc_t *spi_wren, *spi_se, *read_8bits, *write_8bits, *read_multiple_8bits;
    
    //SendSingleCommand(u8 addr)
    spi_wren = &spi_wren_desc;
    // 'MOSI' desc - register addr phase - START_ADDR modified in SendSingleCommand(u8 addr)
    spi_wren->MOSI_regs_addr.CONFIG				= (FLOW_STOP | NDSIZE_0 | WDSIZE_8 | DMAEN);
	spi_wren->MOSI_regs_addr.X_CNT				= 1;
	spi_wren->MOSI_regs_addr.X_MOD				= 0;
	
	// 'MISO' desc - register addr phase - no modifications required
	spi_wren->MISO_regs_addr.START_ADDR_L		= ((u32)(&(spi_wren->dummy_read))  & 0xFFFF);
	spi_wren->MISO_regs_addr.START_ADDR_H		= (((u32)(&(spi_wren->dummy_read)) >> 16) & 0xFFFF);
	spi_wren->MISO_regs_addr.CONFIG				= (FLOW_STOP | NDSIZE_0 | WNR | WDSIZE_8 | DMAEN | DI_EN);
	spi_wren->MISO_regs_addr.X_CNT				= 1;
	spi_wren->MISO_regs_addr.X_MOD				= 0;			
	
	//SectorErase(u8 opcode, u8* addr)
	spi_se = &spi_se_desc;
	// 'MOSI' desc - register addr phase - START_ADDR modified in SectorErase
	spi_se->MOSI_regs_addr.CONFIG			= (FLOW_ARRAY | NDSIZE_5 | WDSIZE_8| DMAEN);
	spi_se->MOSI_regs_addr.X_CNT			= 1;
	spi_se->MOSI_regs_addr.X_MOD			= 0;
	// 'MOSI desc - register value phase - START_ADDR modified in SectorErase
	spi_se->MOSI_regs_value.CONFIG		    = (FLOW_STOP | NDSIZE_0 | WDSIZE_8| DMAEN);
	spi_se->MOSI_regs_value.X_CNT			= 3;
	spi_se->MOSI_regs_value.X_MOD			= 1;
	// 'MISO' desc - register addr phase - no modifications required
	spi_se->MISO_regs_addr.START_ADDR_L		= ((u32)(&(spi_se->dummy_read))  & 0xFFFF);
	spi_se->MISO_regs_addr.START_ADDR_H		= (((u32)(&(spi_se->dummy_read)) >> 16) & 0xFFFF);
	spi_se->MISO_regs_addr.CONFIG			= (FLOW_STOP | NDSIZE_0 | DI_EN | WNR | WDSIZE_8 | DMAEN);
	spi_se->MISO_regs_addr.X_CNT			= 4;
	spi_se->MISO_regs_addr.X_MOD			= 0;			
	
    //sm_spi_read_8bits(u8 addr, u8 *data, u32 len)
	read_8bits = &sm_sport_read_8bits_desc;
	// 'MOSI' desc - register addr phase - START_ADDR modified in sm_spi_read_8bits(u8 addr, u8 *data, u32 len)
	read_8bits->MOSI_regs_addr.CONFIG			= (FLOW_ARRAY | NDSIZE_5 | WDSIZE_8| DMAEN);
	read_8bits->MOSI_regs_addr.X_CNT			= 1;
	read_8bits->MOSI_regs_addr.X_MOD			= 1;
	
	// 'MOSI desc - register value phase - X_CNT modified in sm_spi_read_8bits
	read_8bits->MOSI_regs_value.START_ADDR_L	= ((u32)(&(read_8bits->dummy_write))  & 0xFFFF);
	read_8bits->MOSI_regs_value.START_ADDR_H	= (((u32)(&(read_8bits->dummy_write)) >> 16) & 0xFFFF);
	read_8bits->MOSI_regs_value.CONFIG		    = (FLOW_STOP | NDSIZE_0 | WDSIZE_8| DMAEN);
	read_8bits->MOSI_regs_value.X_MOD			= 0;
	
	// 'MISO' desc - register addr phase - no modifications required
	read_8bits->MISO_regs_addr.START_ADDR_L	    = ((u32)(&(read_8bits->dummy_read))  & 0xFFFF);
	read_8bits->MISO_regs_addr.START_ADDR_H	    = (((u32)(&(read_8bits->dummy_read)) >> 16) & 0xFFFF);
	read_8bits->MISO_regs_addr.CONFIG			= (FLOW_ARRAY | NDSIZE_5 | WNR | WDSIZE_8 | DMAEN);
	read_8bits->MISO_regs_addr.X_CNT			= 1;
	read_8bits->MISO_regs_addr.X_MOD			= 0;
	
	// 'MISO' desc - register value phase - START_ADDR and X_CNT modified in sm_spi_read_8bits
	read_8bits->MISO_regs_value.CONFIG		    = (FLOW_STOP | NDSIZE_0 | DI_EN | WNR | WDSIZE_8 | DMAEN);
	read_8bits->MISO_regs_value.X_MOD			= 1;		
	
	//sm_spi_write_8bits(u8 opcode, u8 *addr, u8 *dataout u32 len, u32 modify)
	write_8bits = &sm_sport_write_8bits_desc;
	
	// 'MOSI' desc - register addr phase - START_ADDR modified in sm_spi_write_8bits
	write_8bits->MOSI_regs_addr.CONFIG			= (FLOW_ARRAY | NDSIZE_5 | WDSIZE_8| DMAEN);
	write_8bits->MOSI_regs_addr.X_CNT			= 1;
	write_8bits->MOSI_regs_addr.X_MOD			= 0;
	
	// 'MOSI desc - register value phase - START_ADDR modified in sm_spi_write_8bits
	write_8bits->MOSI_regs_value.CONFIG		    = (FLOW_ARRAY | NDSIZE_5 | WDSIZE_8| DMAEN);
	write_8bits->MOSI_regs_value.X_CNT			= 3;
	write_8bits->MOSI_regs_value.X_MOD			= 1;
	
	// 'MOSI desc - register value phase - START_ADDR, X_COUNT, X_MOD modified in sm_spi_write_8bits
	write_8bits->MOSI_regs_value2.CONFIG		= (FLOW_STOP | NDSIZE_0 | WDSIZE_8| DMAEN);	
			
	// 'MISO' desc - register addr phase - no modifications required
	write_8bits->MISO_regs_addr.START_ADDR_L	= ((u32)(&(write_8bits->dummy_read))  & 0xFFFF);
	write_8bits->MISO_regs_addr.START_ADDR_H	= (((u32)(&(write_8bits->dummy_read)) >> 16) & 0xFFFF);
	write_8bits->MISO_regs_addr.CONFIG			= (FLOW_ARRAY | NDSIZE_5 | WNR | WDSIZE_8 | DMAEN);
	write_8bits->MISO_regs_addr.X_CNT			= 4;
	write_8bits->MISO_regs_addr.X_MOD			= 0;
	
	// 'MISO' desc - register value phase - X_CNT modified in sm_spi_write_8bits
	write_8bits->MISO_regs_value.START_ADDR_L	= ((u32)(&(write_8bits->dummy_read))  & 0xFFFF);
	write_8bits->MISO_regs_value.START_ADDR_H	= (((u32)(&(write_8bits->dummy_read)) >> 16) & 0xFFFF);
	write_8bits->MISO_regs_value.CONFIG			= (FLOW_STOP | NDSIZE_0 | DI_EN | WNR | WDSIZE_8 | DMAEN);
	write_8bits->MISO_regs_value.X_MOD			= 0;
			
	//sm_spi_read_multiple_8bits(u8 opcode, u8 *addr, u8 *datain, u32 len)
	read_multiple_8bits = &sm_sport_read_multiple_8bits_desc;
	
	// 'MOSI' desc - register addr phase - START_ADDR modified in sm_sport_read_multiple_8bits desc
	read_multiple_8bits->MOSI_regs_addr.CONFIG			= (FLOW_ARRAY | NDSIZE_5 | WDSIZE_8| DMAEN);
	read_multiple_8bits->MOSI_regs_addr.X_CNT			= 1;
	read_multiple_8bits->MOSI_regs_addr.X_MOD			= 0;
	
	// 'MOSI desc - register value phase - START_ADDR modified in sm_sport_read_multiple_8bits desc
	read_multiple_8bits->MOSI_regs_value.CONFIG		    = (FLOW_ARRAY | NDSIZE_5 | WDSIZE_8| DMAEN);
	read_multiple_8bits->MOSI_regs_value.X_CNT			= 3;
	read_multiple_8bits->MOSI_regs_value.X_MOD			= 1;
	
	// 'MOSI desc - register value phase - X_COUNT modified in sm_sport_read_multiple_8bits desc
	read_multiple_8bits->MOSI_regs_value2.START_ADDR_L	= ((u32)(&(read_multiple_8bits->dummy_write))  & 0xFFFF);
	read_multiple_8bits->MOSI_regs_value2.START_ADDR_H	= (((u32)(&(read_multiple_8bits->dummy_write)) >> 16) & 0xFFFF);
	read_multiple_8bits->MOSI_regs_value2.CONFIG		= (FLOW_STOP | NDSIZE_0 | WDSIZE_8| DMAEN);	
	read_multiple_8bits->MOSI_regs_value2.X_MOD			= 0;		
		
	// 'MISO' desc - register addr phase - no modifications required
	read_multiple_8bits->MISO_regs_addr.START_ADDR_L	= ((u32)(&(read_multiple_8bits->dummy_read))  & 0xFFFF);
	read_multiple_8bits->MISO_regs_addr.START_ADDR_H	= (((u32)(&(read_multiple_8bits->dummy_read)) >> 16) & 0xFFFF);
	read_multiple_8bits->MISO_regs_addr.CONFIG			= (FLOW_ARRAY | NDSIZE_5 | WNR | WDSIZE_8 | DMAEN);
	read_multiple_8bits->MISO_regs_addr.X_CNT			= 4;
	read_multiple_8bits->MISO_regs_addr.X_MOD			= 0;
	// 'MISO' desc - register value phase - START_ADDR and X_COUNT modified in sm_sport_read_multiple_8bits desc 
	read_multiple_8bits->MISO_regs_value.CONFIG			= (FLOW_STOP | NDSIZE_0 | DI_EN | WNR | WDSIZE_8 | DMAEN);
	read_multiple_8bits->MISO_regs_value.X_MOD			= 1;	

}// end of Init_Descriptors

//////////////////////////////////////////////////////////////
// ERROR_CODE GetCodes()
//
// Sector Table Setup for a certain device
//
//////////////////////////////////////////////////////////////

ERROR_CODE GetCodes()
{
	// This function makes use of the settings in the file "SPI_Device_Settings.h".
	
	// In regards to the device chosen the sectors will be organized.
	AFP_NumSectors = SPI_SECTORS;
	AFP_SectorSize = SPI_SECTOR_SIZE;

	SectorInfo[0].lStartOff = SECTORBASE;
	SectorInfo[0].lEndOff =   SECTORBASE + SPI_SECTOR_SIZE - 1;
	SectorInfo[1].lStartOff = SECTORBASE + SPI_SECTOR_SIZE;
	SectorInfo[1].lEndOff =   SECTORBASE + 2 * SPI_SECTOR_SIZE - 1;
	SectorInfo[2].lStartOff = SECTORBASE + 2 * SPI_SECTOR_SIZE;
	SectorInfo[2].lEndOff =   SECTORBASE + 3 * SPI_SECTOR_SIZE - 1;
	SectorInfo[3].lStartOff = SECTORBASE + 3 * SPI_SECTOR_SIZE;
	SectorInfo[3].lEndOff =   SECTORBASE + 4 * SPI_SECTOR_SIZE - 1;


	return NO_ERR;
	
}


//////////////////////////////////////////////////////////////
// ERROR_CODE EraseFlash()
//
// Sends an "erase all" command to the flash.
//
//////////////////////////////////////////////////////////////

ERROR_CODE EraseFlash()
{
	
	ERROR_CODE ErrorCode = NO_ERR;
	
	//A write enable instruction must previously have been executed
	SendSingleCommand(SPI_WREN);
 	
	//The status register will be polled to check the write enable latch "WREN"
	ErrorCode = Wait_For_nStatus();

	if( POLL_TIMEOUT == ErrorCode )
	{
		return ErrorCode;
	}
	else

	SendSingleCommand(SPI_BE);
 		
	// Erasing the whole flash will take time, so the following bit must be polled.
	//The status register will be polled to check the write in progress bit "WIP"
	ErrorCode = Wait_For_Status(WIP);
	
	return ErrorCode;
}

//////////////////////////////////////////////////////////////
// ERROR_CODE ResetFlash()
//
// This function sends a Write Disable Command to the SPI device
// After that it checks the Write Enable Latch.
//
//////////////////////////////////////////////////////////////

ERROR_CODE ResetFlash()
{
	char status_register = 0;
	ERROR_CODE ErrorCode = NO_ERR;	
	
	SendSingleCommand(SPI_WRDI);
	ErrorCode = Wait_For_Status(WEL);
	
	// reset should be complete
	return ErrorCode;
}


//////////////////////////////////////////////////////////////
// ERROR_CODE EraseBlock()
//
// Sends an "erase block" command to the flash.
//
//////////////////////////////////////////////////////////////

ERROR_CODE EraseBlock( int nBlock )
{
	unsigned long ulSectorOff = 0x0;
	unsigned char ShiftValue[3];
	ERROR_CODE ErrorCode = NO_ERR;	


	// if the block is invalid "out of possible rangs" just return with an error.
	if ( (nBlock < 0) || (nBlock > AFP_NumSectors) )
	{
		ErrorCode = INVALID_BLOCK;	// tells us if there was an error erasing flash
		return ErrorCode;
	}
	
	// figure out the offset of the block in flash
	if ( (nBlock >= 0) && (nBlock < AFP_NumSectors) )
	{
		ulSectorOff = (nBlock * AFP_SectorSize);
		
	}
	else
	{
		ErrorCode = INVALID_BLOCK;	// tells us if there was an error erasing flash
		return ErrorCode;
	}

	// A write enable instruction must previously have been executed
	SendSingleCommand(SPI_WREN);
	
	//The status register will be polled to check the write enable latch "WREN"
	ErrorCode = Wait_For_nStatus();
	
	if( POLL_TIMEOUT == ErrorCode )
	{
		return ErrorCode;
	}
	else

	// Send the erase block command to the flash followed by the address 
	// to point to the start of a sector.
	ShiftValue[0] = ulSectorOff >> 16;
	ShiftValue[1] = ulSectorOff >> 8;
	ShiftValue[2] = ulSectorOff;	
	SectorErase(SPI_SE,(u8*) &ShiftValue[0]);
	    
	// Poll the status register to check the Write in Progress bit
	// Sector erase takes time
	ErrorCode = Wait_For_Status(WIP);

	// block erase should be complete
	return ErrorCode;
}




//////////////////////////////////////////////////////////////
// ERROR_CODE FillData()
//
// Fill flash with a value.
//
// Inputs:	unsigned long ulStart - offset in flash to start the writes at
//			long lCount - number of elements to write, in this case bytes
//			long lStride - number of locations to skip between writes
//			int *pnData - pointer to data buffer
//
//////////////////////////////////////////////////////////////

ERROR_CODE FillData( unsigned long ulStart, long lCount, long lStride, int *pnData )
{

	
	unsigned long ulWAddr = ulStart; 
	long lWCount = 1, lWriteCount;
	long *pnWriteCount = &lWriteCount;

	int i;
	ERROR_CODE ErrorCode = NO_ERR;

	
    for (i = 0; i < lCount; i++, ulWAddr += lStride)
	{		
		ErrorCode = WriteFlash(ulWAddr, lWCount, lStride, pnData, pnWriteCount);
	}
	
	// return the appropriate error code
	return ErrorCode;
	
}

///////////////////////////////////////////////////////////////////////////////

// ERROR_CODE WriteData()
//
// Write the contents of AFP_Buffer to SPI flash.
//
// Inputs:	unsigned long ulStart - SPI start address 
//			long lCount - number of elements write to the SPI
//			long lStride - address increment to the SPI
//			int *pnData - pointer to data buffer
//
// This function takes the input parameters and calls the function 
// WriteFlash which writes the data of AFP_Buffer to the SPI.
//
// The function WriteFlash will usually be executed multiple times because 
// the SPI needs a break after a SPI page has been filled.
// After each page transfer the addresses and counts need to be recalculated .
///////////////////////////////////////////////////////////////////////////////

ERROR_CODE WriteData( unsigned long ulStart, long lCount, long lStride, int *pnData )
{

	unsigned long ulWStart = ulStart; 
	long lWCount = lCount, lWriteCount;
	long *pnWriteCount = &lWriteCount;
	int i;
	
	
	ERROR_CODE ErrorCode = NO_ERR;

	while (lWCount != 0)	
	{
		ErrorCode = WriteFlash(ulWStart, lWCount, lStride, pnData, pnWriteCount);
		
		// After each function call of WriteFlash the counter must be adjusted
		lWCount -= *pnWriteCount;

#ifdef ATMEL_DATAFLASH
		
		ulWStart += ( *pnWriteCount + PAGE_SIZE_DIFF );


#else		
		// Also, both address pointers must be recalculated.
		ulWStart += *pnWriteCount;

#endif		

		pnData += *pnWriteCount/4;
		
	}

	// return the appropriate error code
	return ErrorCode;
	
}

//////////////////////////////////////////////////////////////
// ERROR_CODE ReadData()
//
// Read a value from flash for verify purpose
//
// Inputs:	unsigned long ulStart - holds the SPI start address
//			int pnData - pointer to store value read from flash
//			long lCount - number of elements to read
//////////////////////////////////////////////////////////////

ERROR_CODE ReadData(  unsigned long ulStart, long lCount, long lStride, int *pnData  )
{
	char *cnData;	
	char ShiftValue [3];	

	cnData = (char *)pnData; // Pointer cast to be able to increment byte wise

	// Send the highest byte of the 24 bit address at first
	ShiftValue[0] = ulStart >> 16;
	// Send the middle byte of the 24 bit address or highest in case or 16 bit addresssing at second
	ShiftValue[1] = ulStart >> 8;
	// Send the lowest byte finally.
	ShiftValue[2] = ulStart;
	// After the SPI device address has been placed on the MOSI pin the data can be
	// received on the MISO pin.
	sm_spi_read_multiple_8bits(SPI_READ, (u8* )&ShiftValue, (u8 *)cnData, lCount);

	return NO_ERR;
}


//////////////////////////////////////////////////////////////
// ERROR_CODE WriteFlash()
//
// Write the maximum of n bytes to flash "limited to the SPI page size".
//
// Inputs:	unsigned long StartAddr - address to write to
//			int iDataSource - pointer to the AFP_Buffer
//			long lTransferCount - number of elements to send
//////////////////////////////////////////////////////////////

ERROR_CODE WriteFlash ( unsigned long ulStartAddr, long lTransferCount, long lModify, int *iDataSource, long *lWriteCount )
{	
	long lWTransferCount = 0;
	int i;
	char iData;
	char dummycommand;
	int dummyread;
	ERROR_CODE ErrorCode = NO_ERR;
	char myShiftRegister[3];

// In case of taking Atmel Data flash the routine below must not be executed.
#ifndef ATMEL_DATAFLASH	

	// First, a Write Enable Command must be sent to the SPI.
	SendSingleCommand(SPI_WREN);
	
	// Second, the SPI Status Register will be tested whether the 
	//         Write Enable Bit has been set. 
	ErrorCode = Wait_For_nStatus();

	if( POLL_TIMEOUT == ErrorCode )
		return ErrorCode;
	else	
#endif	
		
	// Third, the address will be shifted out the SPI MOSI bytewise.
	// Fourth, maximum number of n bytes will be taken from the Buffer
	// and sent to the SPI device. "SPI_PAGE_SIZE" can be found in the file "SPI_Device_Settings.h"

	myShiftRegister[0] = ulStartAddr >> 16;
	myShiftRegister[1] = ulStartAddr >> 8;
	myShiftRegister[2] = ulStartAddr;
	if (lTransferCount <= SPI_PAGE_SIZE) {
		sm_spi_write_8bits(SPI_PP, (u8 *) &myShiftRegister[0], (u8 *) iDataSource, lTransferCount, lModify);	
		*lWriteCount = lTransferCount;
	}
	else {
		sm_spi_write_8bits(SPI_PP, (u8 *) &myShiftRegister[0], (u8 *) iDataSource, SPI_PAGE_SIZE, lModify);
		*lWriteCount = SPI_PAGE_SIZE;
	}    
	//dummycommand = *((char *)iDataSource);
	//((char *)iDataSource)++;
	//sm_spi_write_8bits(dummycommand, (u8* )iDataSource, lTransferCount-1);
	// Sixth, the SPI Write in Progress Bit must be toggled to ensure the 
	// programming is done before start of next transfer.
	
#ifdef ATMEL_DATAFLASH
	ErrorCode = Wait_For_nStatus(WIP);	
#else
	ErrorCode = Wait_For_Status(WIP);
#endif
		
	if( POLL_TIMEOUT == ErrorCode )
		return ErrorCode;
	else
	
	return ErrorCode;
}


//////////////////////////////////////////////////////////////
// ERROR_CODE GetSectorNumber()
//
// Gets a sector number based on the offset.
//
// In the flash programmer plug-in a sector can be chosen
// and than be erased.
//////////////////////////////////////////////////////////////

ERROR_CODE GetSectorNumber( unsigned long ulOffset, int *pnSector )
{
	int nSector = 0;
	int i=0;
	ERROR_CODE ErrorCode = NO_ERR;

	// This function makes use of the settings in the file "SPI_Device_Settings.h".

	// determine the sector
	
	if(ulOffset > SPI_SECTOR_SIZE * SPI_SECTORS - 1)
	{
		ErrorCode = INVALID_SECTOR;
		return ErrorCode;
	}
	
	
	for (i=SPI_SECTORS-1;i>=0;i--) {
		if (ulOffset >= SECTORBASE + i * SPI_SECTOR_SIZE) {
			nSector = i;
			break;
		}
	}
					
	*pnSector = nSector;
	
	// ok
	return ErrorCode;
}



//////////////////////////////////////////////////////////////
// Wait_For_nStatus(void)
//
// Polls the WEL (Write Enable Latch) bit of the Flash's status
// register.
// Inputs - none
// returns- none
//
//////////////////////////////////////////////////////////////

ERROR_CODE Wait_For_nStatus(void)
{
	int n, i;
	char status_register = 0;
	ERROR_CODE ErrorCode = NO_ERR;	// tells us if there was an error erasing flash
	
		for(i = 0; i < 500; i++)
		{
			status_register = ReadStatusRegister();
			if( (status_register & WEL) )
			{
				ErrorCode = NO_ERR;
				return ErrorCode;
			}
			ErrorCode = POLL_TIMEOUT;	// Time out error
		};
							
	return ErrorCode;
}
//////////////////////////////////////////////////////////////
// Wait_For_Status(void)
//
// Polls the Status Register of the Flash's status
// register until the Flash is finished with its access. Accesses
// that are affected by a latency are Page_Program, Sector_Erase,
// and Block_Erase.
// Inputs - Statusbit
// returns- none
//
//////////////////////////////////////////////////////////////

ERROR_CODE Wait_For_Status( char Statusbit )
{
	int n, i;
	char status_register = 0xFF;
	ERROR_CODE ErrorCode = NO_ERR;	// tells us if there was an error erasing flash
	
		for(i = 0; i < 350000; i++) // polls for more than 3 seconds
		{
			status_register = ReadStatusRegister();
			if( !(status_register & Statusbit) )
			{
				ErrorCode = NO_ERR;	// tells us if there was an error erasing flash
				break;
			}

			ErrorCode = POLL_TIMEOUT;	// Time out error
		};
		
	
	return ErrorCode;
	
}

//////////////////////////////////////////////////////////////
// int ReadStatusRegister(void)
//
// Returns the 8-bit value of the status register.
// Inputs - none
// returns- second location of status_register[2],
//         first location is garbage.
// Core sends the command
//
//////////////////////////////////////////////////////////////
char ReadStatusRegister(void)
{
	static char data;
	sm_spi_read_8bits(SPI_RDSR, (u8 *)&data, sizeof(data));
	return data;
}

//Juan's code
void SectorErase(u8 opcode, u8* addr)
{
	static u8 opcode_addr;
	sm_sport_desc_t *spi_se;	

	if(!(*pSPORT1_RCR1 & RSPEN)) {
			
    	opcode_addr = opcode;			
	  	// update descriptors
		spi_se = &spi_se_desc;							
		spi_se->MOSI_regs_addr.START_ADDR_L		= ((u32)&opcode_addr  & 0xFFFF);
		spi_se->MOSI_regs_addr.START_ADDR_H		= (((u32)&opcode_addr >> 16) & 0xFFFF);
		spi_se->MOSI_regs_value.START_ADDR_L	= ((u32)addr  & 0xFFFF);
		spi_se->MOSI_regs_value.START_ADDR_H	= (((u32)addr >> 16) & 0xFFFF);
				
		// now DMA
		*pDMA6_CONFIG 			= 0x0;
		*pDMA6_PERIPHERAL_MAP	= 0x6000;
		*pDMA6_CURR_DESC_PTR	= (void *)&(spi_se->MOSI_regs_addr); 
		*pDMA6_CONFIG 			= (FLOW_ARRAY | NDSIZE_5 | WDSIZE_8 | DMAEN);
	
 		*pDMA5_CONFIG 			= 0x0;
		*pDMA5_PERIPHERAL_MAP	= 0x5000;
		*pDMA5_CURR_DESC_PTR	= (void *)&(spi_se->MISO_regs_addr);
		*pDMA5_CONFIG 			= (FLOW_ARRAY | NDSIZE_5 | WNR | DI_EN | WDSIZE_8 | DMAEN );
		ssync();	

		*pSPORT1_TCR1			|= TSPEN;   	
    	*pSPORT1_RCR1			|= RSPEN;

    	ssync();
	
//    	#if (!defined(asynchronous_SPORT))
	   		while(*pSPORT1_RCR1 & RSPEN);	
//    	#endif	       	
	}
	return;
}


void SendSingleCommand(u8 addr)
{
	static u8 phaser_reg_addr;
	sm_sport_desc_t *spi_wren;	

	if(!(*pSPORT1_RCR1 & RSPEN)) {
	
    	phaser_reg_addr = addr;			
	  	// update descriptors
		spi_wren = &spi_wren_desc;							
		spi_wren->MOSI_regs_addr.START_ADDR_L	= ((u32)&phaser_reg_addr  & 0xFFFF);
		spi_wren->MOSI_regs_addr.START_ADDR_H	= (((u32)&phaser_reg_addr >> 16) & 0xFFFF);
				
		// now DMA
		*pDMA6_CONFIG 			= 0x0;
		*pDMA6_PERIPHERAL_MAP	= 0x6000;
		*pDMA6_CURR_DESC_PTR	= (void *)&(spi_wren->MOSI_regs_addr); 
		*pDMA6_CONFIG 			= (FLOW_ARRAY | NDSIZE_5 | WDSIZE_8 | DMAEN);
	
 		*pDMA5_CONFIG 			= 0x0;
		*pDMA5_PERIPHERAL_MAP	= 0x5000;
		*pDMA5_CURR_DESC_PTR	= (void *)&(spi_wren->MISO_regs_addr);
		*pDMA5_CONFIG 			= (FLOW_ARRAY | NDSIZE_5 | WNR | WDSIZE_8 | DMAEN | DI_EN);
		ssync();	

		*pSPORT1_TCR1			|= TSPEN;   	
    	*pSPORT1_RCR1			|= RSPEN;

    	ssync();
	
//    	#if (!defined(asynchronous_SPORT))
	   		while(*pSPORT1_RCR1 & RSPEN);	
//    	#endif	       	
	}
	return;
}

//--------------------------------------------------------------------------//
// Function:	sm_spi_read_8bits
//																			
// Parameters:	u8  addr :
//				u8 *data :
//				int len	 :
//																			
// Return:		int														
//																			
// Description:	sm_spi_read		 										
//--------------------------------------------------------------------------//
int sm_spi_read_8bits(u8 addr, u8 *data, u32 len) 
{
	static u8 phaser_reg_addr;
	sm_sport_desc_t *read_8bits;
	u8 ret = 1;

	if(!(*pSPORT1_RCR1 & RSPEN))
	{
		phaser_reg_addr = addr;
	
		// update descriptors
		read_8bits = &sm_sport_read_8bits_desc;
		read_8bits->MOSI_regs_addr.START_ADDR_L	 = ((u32)&phaser_reg_addr  & 0xFFFF);
		read_8bits->MOSI_regs_addr.START_ADDR_H	 = (((u32)&phaser_reg_addr >> 16) & 0xFFFF);
		read_8bits->MOSI_regs_value.X_CNT 		 = len;
		read_8bits->MISO_regs_value.START_ADDR_L = ((u32)data  & 0xFFFF);
		read_8bits->MISO_regs_value.START_ADDR_H = (((u32)data >> 16) & 0xFFFF);
		read_8bits->MISO_regs_value.X_CNT 		 = len;
	
		// now DMA
		*pDMA6_CONFIG 			= 0x0;
		*pDMA6_PERIPHERAL_MAP	= 0x6000;
		*pDMA6_CURR_DESC_PTR	= (void *)&(read_8bits->MOSI_regs_addr); 
	   	*pDMA6_CONFIG 			= (FLOW_ARRAY | NDSIZE_5 | WDSIZE_8 | DMAEN);
	
		*pDMA5_CONFIG 			= 0x0;
		*pDMA5_PERIPHERAL_MAP	= 0x5000;
		*pDMA5_CURR_DESC_PTR	= (void *)&(read_8bits->MISO_regs_addr);
	   	*pDMA5_CONFIG 			= (FLOW_ARRAY | NDSIZE_5 | WDSIZE_8 | DMAEN);
		ssync();
	
		 *pSPORT1_TCR1			|= TSPEN;
		 *pSPORT1_RCR1			|= RSPEN;
		 ssync();
	
//#if (!defined(asynchronous_SPORT))
	while(*pSPORT1_RCR1 & RSPEN);
//#endif
	    ret = 0;
	}
	return ret;
}


//--------------------------------------------------------------------------//
// Function:	sm_spi_write_8bits 												
//																			
// Parameters:	u8 addr :
//				u8 *data :
//				int len	 :
//																			
// Return:		int														
//																			
// Description:	sm_spi_write		 										
//--------------------------------------------------------------------------//

//void sm_spi_read_multiple_8bits(u8 opcode, u8 *addr, u8 *datain, u32 len)
void sm_spi_write_8bits(u8 opcode, u8 *addr, u8 *dataout, u32 len, u32 modify) 
{
	static u8 phaser_opcode;
	sm_sport_desc_t *write_8bits;
	
	if (!(*pSPORT1_RCR1 & RSPEN))
	{
		phaser_opcode = opcode;
	
		// update descriptors
		write_8bits = &sm_sport_write_8bits_desc;
		write_8bits->MOSI_regs_addr.START_ADDR_L	= ((u32)&phaser_opcode  & 0xFFFF);		// MOSI - register addr phase
		write_8bits->MOSI_regs_addr.START_ADDR_H	= (((u32)&phaser_opcode >> 16) & 0xFFFF);
		write_8bits->MOSI_regs_value.START_ADDR_L	= ((u32)addr  & 0xFFFF);				// MOSI - register addr phase
		write_8bits->MOSI_regs_value.START_ADDR_H	= (((u32)addr >> 16) & 0xFFFF);
		write_8bits->MOSI_regs_value2.START_ADDR_L	= ((u32)dataout  & 0xFFFF);				// MOSI - register addr phase
		write_8bits->MOSI_regs_value2.START_ADDR_H	= (((u32)dataout >> 16) & 0xFFFF);
		write_8bits->MOSI_regs_value2.X_CNT			= len;		
		write_8bits->MOSI_regs_value2.X_MOD			= modify;													
		write_8bits->MISO_regs_value.X_CNT 			= len;			
	 	
	 	
		// now DMA
		*pDMA6_CONFIG 			= 0x0;
		*pDMA6_PERIPHERAL_MAP	= 0x6000;
		*pDMA6_CURR_DESC_PTR	= (void *)&(write_8bits->MOSI_regs_addr);
	   	*pDMA6_CONFIG 			= (FLOW_ARRAY | NDSIZE_5 | WDSIZE_8 | DMAEN);
	
		*pDMA5_CONFIG 			= 0x0;
		*pDMA5_PERIPHERAL_MAP	= 0x5000;
		*pDMA5_CURR_DESC_PTR	= (void *)&(write_8bits->MISO_regs_addr);
	   	*pDMA5_CONFIG 			= FLOW_ARRAY | NDSIZE_5 | DI_EN | WNR | WDSIZE_8 | DMAEN;
		ssync();
	
		*pSPORT1_TCR1			|= TSPEN;
		*pSPORT1_RCR1			|= RSPEN;
		ssync();
	
//#if (!defined(asynchronous_SPORT))
	while(*pSPORT1_RCR1 & RSPEN);
//#endif
		
	}
	
	return;
}


void sm_spi_read_multiple_8bits(u8 opcode, u8 *addr, u8 *datain, u32 len)
{
	static u8 phaser_opcode;
	sm_sport_desc_t *read_multiple_8bits;
	
	if (!(*pSPORT1_RCR1 & RSPEN))
	{
		phaser_opcode = opcode;	
			
		// update descriptors
		read_multiple_8bits = &sm_sport_read_multiple_8bits_desc;
		// 'MOSI' desc - register addr phase - START_ADDR modified in sm_sport_read_multiple_8bits desc
		read_multiple_8bits->MOSI_regs_addr.START_ADDR_L	= ((u32)(&(phaser_opcode))  & 0xFFFF);
		read_multiple_8bits->MOSI_regs_addr.START_ADDR_H	= (((u32)(&(phaser_opcode)) >> 16) & 0xFFFF);		
		// 'MOSI desc - register value phase - START_ADDR modified in sm_sport_read_multiple_8bits desc
		read_multiple_8bits->MOSI_regs_value.START_ADDR_L	= ((u32)(addr)  & 0xFFFF);
		read_multiple_8bits->MOSI_regs_value.START_ADDR_H	= (((u32)(addr) >> 16) & 0xFFFF);					
	
		// 'MOSI desc - register value phase - X_COUNT modified in sm_sport_read_multiple_8bits desc
		// 'MISO' desc - register value phase - START_ADDR and X_COUNT modified in sm_sport_read_multiple_8bits desc 
		#ifdef ATMEL_DATAFLASH   //4 dummy reads are required
			read_multiple_8bits->MOSI_regs_value2.X_CNT			= len+4;			
			read_multiple_8bits->MISO_regs_addr.X_CNT			= 8;
		else
			read_multiple_8bits->MOSI_regs_value2.X_CNT			= len;	
		#endif							
		read_multiple_8bits->MISO_regs_value.START_ADDR_L	= ((u32)(datain)  & 0xFFFF);
		read_multiple_8bits->MISO_regs_value.START_ADDR_H	= (((u32)(datain) >> 16) & 0xFFFF);				
		read_multiple_8bits->MISO_regs_value.X_CNT			= len;		
	
		// now DMA
		*pDMA6_CONFIG 			= 0x0;
		*pDMA6_PERIPHERAL_MAP	= 0x6000;
		*pDMA6_CURR_DESC_PTR	= (void *)&(read_multiple_8bits->MOSI_regs_addr);
	   	*pDMA6_CONFIG 			= (FLOW_ARRAY | NDSIZE_5 | WDSIZE_8 | DMAEN);
	
		*pDMA5_CONFIG 			= 0x0;
		*pDMA5_PERIPHERAL_MAP	= 0x5000;
		*pDMA5_CURR_DESC_PTR	= (void *)&(read_multiple_8bits->MISO_regs_addr);
	   	*pDMA5_CONFIG 			= FLOW_ARRAY | NDSIZE_5 | WNR | WDSIZE_8 | DMAEN;
		ssync();
	
		*pSPORT1_TCR1			|= TSPEN;
		*pSPORT1_RCR1			|= RSPEN;
		ssync();
	
//#if (!defined(asynchronous_SPORT))
	while(*pSPORT1_RCR1 & RSPEN);
//#endif
		
	}
	return;
}


//--------------------------------------------------------------------------//
// Function:	SPORT1_ISR													//
//																			//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description:	SPORT1_ISR													//
//--------------------------------------------------------------------------//
EX_INTERRUPT_HANDLER(SPORT1_ISR)
{
	if (*pDMA5_IRQ_STATUS & DMA_DONE)
		*pDMA5_IRQ_STATUS 	|= DMA_DONE;				// Signal that interrupt has been serviced
	
	if (*pDMA6_IRQ_STATUS & DMA_DONE)
		*pDMA6_IRQ_STATUS 	|= DMA_DONE;				// Signal that interrupt has been serviced
					
	 *pSPORT1_TCR1			&= ~(TSPEN);
	 *pSPORT1_RCR1			&= ~(RSPEN);
	 ssync();

}/* End of SPORT1_ISR */

//--------------------------------------------------------------------------//
// Function:	SYS_ERR_ISR 												//
//																			//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description:	Generic SYS_ERR ISR 										//
//--------------------------------------------------------------------------//
EX_INTERRUPT_HANDLER(SYS_ERR_ISR)
{
	if (*pSPORT1_STAT & (TUVF|TOVF))
		*pSPORT1_STAT |= (TUVF|TOVF);				// clear SPORT0 Errors

	if (*pDMA5_IRQ_STATUS & DMA_ERR)
		*pDMA5_IRQ_STATUS |= DMA_ERR;				// SPORT - Signal that interrupt has been serviced	
	if (*pDMA6_IRQ_STATUS & DMA_ERR)
		*pDMA6_IRQ_STATUS |= DMA_ERR;				// SPORT - Signal that interrupt has been serviced	

	ssync();
	while(1);
	
	//while(1);										// wait here 
}/* End of SYS_ERR_ISR */

//--------------------------------------------------------------------------//
// Function:	HW_ERR_ISR													//
//																			//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description:	Generic HW_ERR ISR											//
//--------------------------------------------------------------------------//
EX_INTERRUPT_HANDLER(HW_ERR_ISR)
{
	while(1);										// wait here
}/* End of HW_ERR_ISR */

//--------------------------------------------------------------------------//
// Function:	EXCEP_ISR													//
//																			//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description:	Generic EXCEPTION ISR										//
//--------------------------------------------------------------------------//
EX_INTERRUPT_HANDLER(EXCEP_ISR)
{	
	while(1);										// wait here
}/* End of EXCEP_ISR */



